#!/usr/bin/python
"""
    Space Parcels -- menu routines
    Copyright (C) 2005 Mathieu "Zig" Brethes <just.want.to.be@free.fr>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
# Class for showing and handling the various menus of the game.

import pygame, os
from pygame.locals import *
from sound import SoundEngine
import myglobals

# Base class for a menu.
class Menu:
	menuIndexor = None
	def __init__(self, parentSurface, image, pos, listPoses):
		if not Menu.menuIndexor:
			Menu.menuIndexor = pygame.image.load(os.path.join("media","images","menu_sel.png")).convert()
			Menu.menuIndexor.set_colorkey(0, RLEACCEL)
		self.screen = parentSurface
		self.image  = image
		self.image.set_colorkey(0)
		self.listPoses = listPoses
		self.pos = pos
		self.getOut = False

	def show(self):
		# we take control!
		sav = pygame.Surface(self.screen.get_rect().size)
		sav.blit(self.screen, [0,0])
		self.screen.blit(self.image, self.pos)
		index = 0
		if self.listPoses:
			self.screen.blit(Menu.menuIndexor, (self.listPoses[index][0][0]+self.pos[0],self.listPoses[index][0][1]+self.pos[1]))
		pygame.display.flip()

		# to be sure nothing happens...
		pygame.event.clear()
		self.getOut = False
		C = pygame.time.Clock()
		C.tick()
		while not self.getOut:
			pygame.event.pump()
			for e in pygame.event.get():
				if e.type == KEYDOWN:
					if e.key == K_ESCAPE:
						self.getOut = True
					elif self.listPoses:
						if e.key == K_DOWN:
							self.screen.fill([3,3,3], pygame.Rect((self.listPoses[index][0][0]+self.pos[0],self.listPoses[index][0][1]+self.pos[1]), (20,20)))
							index =  (index + 1) % len(self.listPoses)
							self.screen.blit(Menu.menuIndexor, (self.listPoses[index][0][0]+self.pos[0],self.listPoses[index][0][1]+self.pos[1]))		
						elif e.key == K_UP:
							self.screen.fill([3,3,3], pygame.Rect((self.listPoses[index][0][0]+self.pos[0],self.listPoses[index][0][1]+self.pos[1]), (20,20)))
							index =  (index - 1) % len(self.listPoses)
							self.screen.blit(Menu.menuIndexor, (self.listPoses[index][0][0]+self.pos[0],self.listPoses[index][0][1]+self.pos[1]))		
						elif e.key == K_RETURN:
							self.listPoses[index][1]()
			pygame.display.flip()
			C.tick(30)		

		# restore old.
		self.screen.blit(sav, [0,0])
		pygame.display.flip()

# class for the bar.
class Bar:
	tBar = None
	tCtl = None
	
	def __init__(self, parentSurface, pos, mini, maxi, cur):
		if not Bar.tBar:
			Bar.tBar = pygame.image.load(os.path.join("media","images","scroll_bg.png")).convert()
		if not Bar.tCtl:
			Bar.tCtl = pygame.image.load(os.path.join("media","images","scroll_bit.png")).convert()
			Bar.tCtl.set_colorkey(0, RLEACCEL)
		self.mini = mini
		self.maxi = maxi
		rect = Bar.tBar.get_rect()
		self.pos = pos
		self.width = rect.width
		self.initVal = cur
		self.val = cur
		self.parentSurface = parentSurface
		self.image = pygame.Surface(rect.size)
		self.image.blit(Bar.tBar,[0,0])
		self.image.blit(Bar.tCtl,[(rect.width - 10 )* (cur - mini)/ (maxi - mini), 0])
		parentSurface.blit(self.image,pos)
	
	def doYourStuff(self, callback):
		C = pygame.time.Clock()
		C.tick()
		pygame.event.clear()
		getOut = False
		move = 0
		while not getOut:
     			self.image.fill([3,3,3], pygame.Rect([(self.width - 10)* (self.val - self.mini)/ (self.maxi - self.mini)+1, 1], (8, 18)))
                        mustCB = False
			for e in pygame.event.get():
				if e.type == KEYDOWN:
					if e.key == K_LEFT:
                                                move = -1
					elif e.key == K_RIGHT:
                                                move = 1
					elif e.key == K_RETURN:
                                                move = 0
						getOut = True
                                                mustCB = True
					elif e.key == K_ESCAPE:
                                                move = 0
						self.val = self.initVal
						getOut = True
                                                mustCB = True
                                elif e.type == KEYUP:
                                        if e.key in [K_LEFT,K_RIGHT]:
                                                move = 0
                        if move != 0:
                                self.val += move
                                if self.val < self.mini:
                                    self.val = self.mini
                                elif self.val > self.maxi:
                                    self.val = self.maxi
                                else:
                                    mustCB = True

                        #print self.val
			self.image.blit(Bar.tCtl,[(self.width -10)* (self.val - self.mini)/ (self.maxi - self.mini), 0])
			self.parentSurface.blit(self.image,self.pos)
#			pygame.display.flip()
                        if mustCB:
                                callback(self.val)

			C.tick(self.maxi - self.mini)

# Class for the sound menu.				
class SoundMenu(Menu):
	SndPic = None
	
	def __init__(self, parent, pos, soundVol, musicVol):
		if not SoundMenu.SndPic:
			SoundMenu.SndPic = pygame.image.load(os.path.join("media","images","menusound.png")).convert()			
			SoundMenu.SndPic.set_colorkey(0)
		self.image = pygame.Surface(SoundMenu.SndPic.get_rect().size)
		self.image.blit(SoundMenu.SndPic, (0,0))
		self.parent = parent
		self.pos = pos
		
		self.bar1 = Bar(self.image, (25, 77), 0, 20, musicVol * 20)
		self.bar2 = Bar(self.image, (25, 137), 0, 20, soundVol * 20)
		listPoses = [((40, 42), lambda: self.barRunSnd(self.bar1)),((40,107), lambda: self.barRunMus(self.bar2))]
		Menu.__init__(self, parent, self.image, pos, listPoses)

	def barRunSnd(self, bar):
		bar.doYourStuff(lambda x:self.setMusicVolume(x))

	def barRunMus(self, bar):
		bar.doYourStuff(lambda x:self.setSoundVolume(x))
                       
	def setSoundVolume(self, val):
		global mySound
		mySound.setVolumes(val/20.0, mySound.getMusicVolume())
		self.parent.blit(self.image, self.pos)
		#print "foo"
                mySound.playBXCrash()
		pygame.display.flip()

	def setMusicVolume(self, val):
		global mySound
		mySound.setVolumes(mySound.getSoundVolume(), val/20.0)
		self.parent.blit(self.image, self.pos)
		#print "bar"
		pygame.display.flip()

# Class for the control menu
class CtlMenu(Menu):
	CtlPic = None

	def __init__(self, parent, pos, controls):
		if not CtlMenu.CtlPic:
			CtlMenu.CtlPic = pygame.image.load(os.path.join("media","images","menuctrl.png")).convert()			
			CtlMenu.CtlPic.set_colorkey(0)
		self.image = pygame.Surface(CtlMenu.CtlPic.get_rect().size)
		self.image.blit(CtlMenu.CtlPic, (0,0))
		self.parent = parent
		self.pos    = pos

		self.controls = controls
		listPoses = [((20, 55), lambda: self.setKey("up")),
			     ((20, 76), lambda: self.setKey("down")),
			     ((20, 97), lambda: self.setKey("left")),
			     ((20, 118), lambda: self.setKey("right")),
			     ((20, 139), lambda: self.setKey("shoot")),
			     ((20, 160), lambda: self.setKey("release"))]

		self.myFont = pygame.font.Font("afont.ttf", 20)
		k = self.myFont.render(pygame.key.name(controls["up"]), True, [255,255,255])
		self.image.blit(k, (191,52))
		k = self.myFont.render(pygame.key.name(controls["down"]), True, [255,255,255])
		self.image.blit(k, (191,73))
		k = self.myFont.render(pygame.key.name(controls["left"]), True, [255,255,255])
		self.image.blit(k, (191,94))
		k = self.myFont.render(pygame.key.name(controls["right"]), True, [255,255,255])
		self.image.blit(k, (191,115))
		k = self.myFont.render(pygame.key.name(controls["shoot"]), True, [255,255,255])
		self.image.blit(k, (191,136))
		k = self.myFont.render(pygame.key.name(controls["release"]), True, [255,255,255])
		self.image.blit(k, (191,157))

		Menu.__init__(self, parent, self.image, pos, listPoses)

	def getKeys(self):
		return self.controls

	def setKey(self, index):
		pygame.event.clear()
		getOut = False
		while not getOut:
			for e in pygame.event.get():
				if e.type == KEYDOWN:
					if e.key != K_ESCAPE:
						self.controls[index] = e.key
					getOut = True
		self.image.blit(CtlMenu.CtlPic, [0,0])
		k = self.myFont.render(pygame.key.name(self.controls["up"]), True, [255,255,255])
		self.image.blit(k, (191,52))
		k = self.myFont.render(pygame.key.name(self.controls["down"]), True, [255,255,255])
		self.image.blit(k, (191,73))
		k = self.myFont.render(pygame.key.name(self.controls["left"]), True, [255,255,255])
		self.image.blit(k, (191,94))
		k = self.myFont.render(pygame.key.name(self.controls["right"]), True, [255,255,255])
		self.image.blit(k, (191,115))
		k = self.myFont.render(pygame.key.name(self.controls["shoot"]), True, [255,255,255])
		self.image.blit(k, (191,136))
		k = self.myFont.render(pygame.key.name(self.controls["release"]), True, [255,255,255])
		self.image.blit(k, (191,157))
		self.parent.blit(self.image, self.pos)
		pygame.display.flip()

# Class for the level menu.				
class LevelMenu(Menu):
	LvlPic = None
	
	def __init__(self, parent, pos, levelOptions):
		if not LevelMenu.LvlPic:
			LevelMenu.LvlPic = pygame.image.load(os.path.join("media","images","menulevel.png")).convert()			
			LevelMenu.LvlPic.set_colorkey(0)
		self.image = pygame.Surface(LevelMenu.LvlPic.get_rect().size)
		self.image.blit(LevelMenu.LvlPic, (0,0))
		self.parent = parent
		self.pos = pos
		self.levelOptions = levelOptions
		
		self.bar1 = Bar(self.image, (20, 83), 1, 3, levelOptions["stars"])
		self.bar2 = Bar(self.image, (20, 149), 1, 1000, levelOptions["boxprob"])
		self.bar3 = Bar(self.image, (20, 213), 30, 300, levelOptions["lenplay"])
		self.bar4 = Bar(self.image, (20, 280), 20, 300, levelOptions["portalsize"])
		listPoses = [((30, 55), lambda: self.goBar1(self.bar1)),
				((30, 118), lambda: self.goBar2(self.bar2)),
				((30, 181), lambda: self.goBar3(self.bar3)),
				((30, 245), lambda: self.goBar4(self.bar4))]
		Menu.__init__(self, parent, self.image, pos, listPoses)

	def goBar1(self, bar):
		bar.doYourStuff(lambda x:self.setOption("stars",x))
	def goBar2(self, bar):
		bar.doYourStuff(lambda x:self.setOption("boxprob",x))
	def goBar3(self, bar):
		bar.doYourStuff(lambda x:self.setOption("lenplay",x))
	def goBar4(self, bar):
		bar.doYourStuff(lambda x:self.setOption("portalsize",x))

	def setOption(self, index, val):
		self.levelOptions[index] = val
		self.parent.blit(self.image, self.pos)
		pygame.display.flip()

	def getOptions(self):
		return self.levelOptions

# and last but not least... The main menu!
class MainMenu(Menu):
	Image = None
	def __init__(self, parent, fromGame, keys, levelOpts):
		global DX, DY, screen, background,mySound	

		DX=myglobals.DX
		DY=myglobals.DY
		screen=myglobals.screen
		background=myglobals.background
		mySound=myglobals.mySound
		if not MainMenu.Image:
			MainMenu.Image = pygame.image.load(os.path.join("media","images","mainmenu.png")).convert()			
			MainMenu.Image.set_colorkey(0)
		self.image = pygame.Surface(MainMenu.Image.get_rect().size)
		self.image.blit(MainMenu.Image, (0,0))
		self.parent = parent
		self.pos = (165,140)
		self.start = False
		self.quit  = False
		self.keys = keys
		self.levelOpts = levelOpts
		if not fromGame:
			listPoses = [((42,41), lambda: self.goforit()),
					  ((42,62), lambda: self.soundOpt()),
					  ((42, 83), lambda: self.ctlOpt()),
					  ((42, 104), lambda: self.levelOpt()),
					  ((42, 125), lambda: self.byebye())]
		else:	
			listPoses = [((42,62), lambda: self.soundOpt()),
					  ((42, 83), lambda: self.ctlOpt()),
						((42, 125), lambda: self.byebye())]

		Menu.__init__(self, parent, self.image, self.pos, listPoses)

	def goforit(self):
		self.start = True
		self.getOut = True

	def byebye(self):
		self.quit = True
		self.getOut = True

	def soundOpt(self):
		global mySound
		S = SoundMenu(self.parent, (70, 140), mySound.getSoundVolume(), mySound.getMusicVolume())
		S.show()

	def ctlOpt(self):
		C= CtlMenu(self.parent, (155, 140), self.keys)
		C.show()
		self.keys = C.getKeys()

	def levelOpt(self):
		L = LevelMenu(screen, (70, 65), self.levelOpts)
		L.show()
		self.levelOpts = L.getOptions()
	
	def getAll(self):
		return (self.keys, self.levelOpts)	

	def hasQuit(self):
		return self.quit	

	def hasStarted(self):
		return self.start

# High score class
class ScoreMenu(Menu):
	Image = None
	def __init__(self, parent, scoreinfo, newscore):
		if not ScoreMenu.Image:
			ScoreMenu.Image = pygame.image.load(os.path.join("media","images","hiscores.png")).convert()	
			ScoreMenu.Image.set_colorkey(0)

		self.image = pygame.Surface(ScoreMenu.Image.get_rect().size)
		self.image.blit(ScoreMenu.Image, (0,0))
		self.parent = parent
		self.pos = (110, 90)
		myFont = pygame.font.Font("afont.ttf", 20)
		pos = 60
		r = myFont.render("Score: %03d in %03.1f sec. with %d stars."%(newscore), True, [255,0,0],[3,3,3])
		self.image.blit(r, (30, 40))			
		for s in scoreinfo:
			#print s
			r = myFont.render("Score: %03d in %03.1f sec. with %d stars."%(s), True, [255,255,255],[3,3,3])
			self.image.blit(r, (30, pos))
			pos += 20

		self.myScores = scoreinfo
		self.myScores = [newscore] + self.myScores[0:10]

		Menu.__init__(self, parent, self.image, self.pos, [])			

	def getScores(self):
		return self.myScores
