#!/usr/bin/python
"""
    Space Parcels -- sound engine
    Copyright (C) 2005 Mathieu "Zig" Brethes <just.want.to.be@free.fr>

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
"""
# basic sound engine. Plays sounds when asked... Knows which sounds to play.

import pygame, os.path
from pygame.locals import *

class SoundEngine:
    SPCrash = None
    SPFire  = None
    SPMove  = None
    CNLoad  = None
    GBOut   = None
    GBIn    = None
    GBMax   = None
    GBGrab  = None
    BXApp   = None
    BXCrash = None
    BXThru  = None
    
    def __init__(self, soundVolume=1.0, musicVolume=1.0):
        if not pygame.mixer.get_init():
            print "No sound?"
            self.Initialized = False
        else:
            self.Initialized = True
            if not SoundEngine.SPCrash:
                SoundEngine.SPCrash = pygame.mixer.Sound(os.path.join("media","sounds","buzzer2.wav"))
                SoundEngine.SPFire  = pygame.mixer.Sound(os.path.join("media","sounds","metal2.wav"))
                SoundEngine.SPMove  = pygame.mixer.Sound(os.path.join("media","sounds","electricalcurrent.wav"))
                SoundEngine.CNLoad  = pygame.mixer.Sound(os.path.join("media","sounds","laserfire2.wav"))
                SoundEngine.GBOut   = pygame.mixer.Sound(os.path.join("media","sounds","muzzleloadshot.wav"))
                SoundEngine.GBIn    = pygame.mixer.Sound(os.path.join("media","sounds","cockgun2.wav"))
                SoundEngine.GBMax   = pygame.mixer.Sound(os.path.join("media","sounds","emptychamber.wav"))
                SoundEngine.GBGrab  = pygame.mixer.Sound(os.path.join("media","sounds","loadclip.wav"))
                SoundEngine.BXApp   = pygame.mixer.Sound(os.path.join("media","sounds","splat.wav"))
                SoundEngine.BXCrash = pygame.mixer.Sound(os.path.join("media","sounds","pop.wav"))
                SoundEngine.BXThru  = pygame.mixer.Sound(os.path.join("media","sounds","buzzer4.wav"))

            self.playingTune = False
            self.SV = soundVolume
            self.MV = musicVolume

    def playTune(self, music):
        if self.Initialized:
            if self.playingTune:
                pygame.mixer.music.fadeout(2000)
            pygame.mixer.music.set_volume(self.MV)
            pygame.mixer.music.load(os.path.join("media","music",music))
            pygame.mixer.music.play(-1)

    
    def playSPCrash(self):
        if self.Initialized:
            SoundEngine.SPCrash.set_volume(self.SV)
            SoundEngine.SPCrash.play()

    def playSPFire(self):
        if self.Initialized:
            SoundEngine.SPFire.set_volume(self.SV)
            SoundEngine.SPFire.play()

    def playSPMove(self):
        if self.Initialized:
            SoundEngine.SPMove.set_volume(self.SV)
            SoundEngine.SPMove.play(-1)

    def stopSPMove(self):
        if self.Initialized:
            SoundEngine.SPMove.fadeout(100)

    def playCNLoad(self):
        if self.Initialized:
            SoundEngine.CNLoad.set_volume(self.SV)
            SoundEngine.CNLoad.play(-1)

    def stopCNLoad(self):
        if self.Initialized:
            SoundEngine.CNLoad.fadeout(100)

    def playGBOut(self):
        if self.Initialized:
            SoundEngine.GBOut.set_volume(self.SV)
            SoundEngine.GBOut.play()

    def playGBIn(self):
        if self.Initialized:
            SoundEngine.GBIn.set_volume(self.SV)
            SoundEngine.GBIn.play()

    def playGBMax(self):
        if self.Initialized:
            SoundEngine.GBMax.set_volume(self.SV)
            SoundEngine.GBMax.play()

    def playGBGrab(self):
        if self.Initialized:
            SoundEngine.GBGrab.set_volume(self.SV)
            SoundEngine.GBGrab.play()

    def playBXApp(self):
        if self.Initialized:
            SoundEngine.BXApp.set_volume(self.SV)
            SoundEngine.BXApp.play()
    
    def playBXCrash(self):
        if self.Initialized:
            SoundEngine.BXCrash.set_volume(self.SV)
            SoundEngine.BXCrash.play()

    def playBXThru(self):
        if self.Initialized:
            SoundEngine.BXThru.set_volume(self.SV)
            SoundEngine.BXThru.play()

    def setVolumes(self,SV, MV):
        if self.Initialized:
            self.SV=SV
            self.MV=MV
            pygame.mixer.music.set_volume(self.MV)

    def getMusicVolume(self):
	if self.Initialized:
            return self.MV
	else: return 0

    def getSoundVolume(self):
	if self.Initialized:
            return self.SV
	else: return 0

    def isInitialized(self):
	return self.Initialized
